// funktioniert mit testRs232_20241219a.prj
// board NodeMCU 1.0 (ESP - 12E Module)

//http://automatedhome.party/2017/04/17/connect-an-inexpensive-oled-display-to-a-esp8266wemos-d1-mini/

  //  Uno            MCP42100(SPI kommunikation)  d1mini          ESP32DevkitCV2  NodeMCULuaV2
  //  -------------------------
  //  D2            CS  (pin 1)                    D4 (GPIO2)      GPIO5          D8 (GPIO 15)
  //  D11 (MOSI)    SI  (pin 3)                    D7 (GPIO13)     GPIO23         MOSI: D7 (GPIO 13) 
  //  D13 (SCK)     SCK (pin 2)                    D5 (GPIO14)     GPIO18         SCK: D5 (GPIO 14)


#define NUM_RELAYS 2

#include <SoftwareSerial.h>
// SoftwareSerial MySerial(4, 5); // RX, TX      // *** mit <SoftwareSerial.h>

#include <SPI.h>   //MCP4210 

//MCP4210 - start
const byte csPin           = 15;       // 15 = D8 = MCP42100 chip select pin1
const byte sckPin          = 14;       // 14 = D5 = MCP42100 chip serial clock pin2
// const byte misoPin           = D6;       // MCP42100 chip not required (master in slave out)
const byte mosiPin         = 13;       // 13 = D7 = MCP42100 chip SI pin3 (master out slave in) 

const int  maxPositions    = 256;     // wiper can move from 0 to 255 = 256 positions
const long rAB             = 10000;  // 10k pot resistance between terminals A and B
const byte rWiper          = 52;     // 125 ohms pot wiper resistance
const byte pot0            = 0x11;    // pot0 addr
const byte pot1            = 0x12;    // pot1 addr
const byte potBoth         = 0x13;    // pot0 and pot1 simultaneous addr
const byte pot0Shutdown    = 0x21;    // pot0 shutdown
const byte pot1Shutdown    = 0x22;    // pot1 shutdown
const byte potBothShutdown = 0x23;    // pot0 and pot1 simultaneous shutdown
//MCP4210 - ende

// Globale Variablen
int relayGPIOs[NUM_RELAYS] = {2, 16};  //pin fuer relais festlegen *** ACHTUNG mit <SoftwareSerial.h> 4 u 5 loschen!

char inputBuffer[10];        // Puffer für die Eingabezeichen
byte bufferIndex = 0;        // Index für den Eingabepuffer
int numbers[2] = {0, 0};     // Array für die zwei Integer-Werte
int *bitArray;
int ii = 0;

unsigned long myTimerMillis = 0; 
int oldNumbers[2] = {0, 0};

void setup() {
  // MySerial.begin(9600); // *** mit <SoftwareSerial.h>
  Serial.begin(9600);
 
// Relais belegen
  for (int i = 0; i < NUM_RELAYS; i++) {
    pinMode(relayGPIOs[i], OUTPUT);
  }
// fuer MPC4210
  digitalWrite(csPin, HIGH);        // MPC4210 chip select default to de-selected
  pinMode(csPin, OUTPUT);           // MPC4210 configure chip select as output
  SPI.begin();
}

void loop() {
  // Zahlenarray aus der Funktion holen
  int *values = readUSBinVec();

  // Werte nur ausgeben, wenn tatsächlich etwas gelesen wurde
  if (values != nullptr) {
    //Serial.print("Erste Zahl: ");
    //Serial.println(values[0]);
    values = CheckForChange(values); 
    setRelayFromInt(values[0]);  //relais setzen 
    //Serial.print("Zweite Zahl: ");
    //Serial.println(values[1]);
    setPotWiper(pot1, values[1]);   //send to mpc4210 
  }
}

int *readUSBinVec() {
  if (Serial.available()) {
  // if (MySerial.available()) { // *** mit <SoftwareSerial.h>
    char receivedChar = Serial.read();  // Einzelnes Zeichen lesen
    // char receivedChar = MySerial.read();  // Einzelnes Zeichen lesen  // *** mit <SoftwareSerial.h>
    //Serial.print("Wert: ");
    //Serial.println(receivedChar);

    if (receivedChar != '>') {
      // Zeichen in den Puffer speichern, solange er nicht voll ist
      if (bufferIndex < sizeof(inputBuffer) - 1) {
        inputBuffer[bufferIndex++] = receivedChar;
        inputBuffer[bufferIndex] = '\0';  // Nullterminator hinzufügen
      }
    } else {
      // Das Zeichen '>' wurde erkannt – jetzt parsen
      //Serial.print("Gelesener String: ");
      Serial.println(inputBuffer);  //zurueck schicken 
      // MySerial.println(inputBuffer); //zurueck schicken   // *** mit <SoftwareSerial.h>

      // Puffer parsen und Zahlen extrahieren
      byte i = 0;
      char *token = strtok(inputBuffer, ";");
      while (token != NULL && i < 2) {
        numbers[i] = atoi(token);  // Token zu Integer konvertieren und speichern
        i++;
        token = strtok(NULL, ";");
      }

      // Puffer zurücksetzen
      bufferIndex = 0;
      memset(inputBuffer, 0, sizeof(inputBuffer));  // Puffer löschen

      // Rückgabe des Arrays mit den Zahlen
      return numbers;
    }
  }

  // Rückgabe von nullptr, wenn keine neuen Werte gelesen wurden
  return nullptr;
}

// Funktion zur Steuerung der Relais basierend auf einem int-Wert
void setRelayFromInt(int value) {
  for (int i = 0; i < NUM_RELAYS; i++) {
    //int bitState = (value >> (NUM_RELAYS - 1 - i)) & 1;  // Extrahiere das i-te Bit
    int bitState = (value >> i) & 1;  // Bits in umgekehrter Reihenfolge auslesen
    digitalWrite(relayGPIOs[i], !bitState);               // Setze den Zustand des Relais
    //Serial.print(bitState);
  }
  //Serial.println();
}

void setPotWiper(int addr, int pos) {

  pos = constrain(pos, 0, 255);            // limit wiper setting to range of 0 to 255

  digitalWrite(csPin, LOW);                // select chip
  SPI.transfer(addr);                      // configure target pot with wiper position
  SPI.transfer(pos);
  digitalWrite(csPin, HIGH);               // de-select chip
}

//has the variable changed since last time here ?
int* CheckForChange(int numbers[2]) {
    int maxTime = 60ul; //[sec]

    // Check if either value in the array has changed
    if (oldNumbers[0] != numbers[0] || oldNumbers[1] != numbers[1]) {
        // Restart the TIMER
        myTimerMillis = millis();

        // Update to the new state
        oldNumbers[0] = numbers[0];
        oldNumbers[1] = numbers[1];
    }

    // Has the TIMER expired?
    if (millis() - myTimerMillis >= (maxTime * 1000ul)) {
        // Set numbers array to {255, 255} if timer has expired
        numbers[0] = 0;
        numbers[1] = 255;
    }

    return numbers;
}
